/* mtquery.cc
 * This file is part of mt-utils library
 * Copyright (c) 2004 by Stanislav Ievlev
 *
 * This file is covered by the GNU Library General Public License,
 * which should be included with mt-utils as the file COPYING.
 */

#include <iostream>
#include <stdexcept>

#include <getopt.h>

#include "linguas.hh"
#include "translation.hh"
#include "singleton.hh"
#include "config.hh"
#include "file_map.hh"
#include "path.hh"
#include "lgk.hh"
#include "str.hh"

#include  "converter.hh"


typedef mt::singleton_array<converter,mt::lang_code> conv_type;
typedef mt::singleton_array<mt::file_map> txtdb_type;

struct compare_names
{
    compare_names(const std::string& from,const std::string& to):
	from_(from),
	to_(to)
	{}
    bool operator()(const mt::lang_pair& lng1,const mt::lang_pair& lng2)
    {
	return distance(lng1) < distance(lng2);
    }
    int distance(const mt::lang_pair& lng)
    {
	 std::string from_name=mt::lang_name(lng.first);
	 std::string to_name=mt::lang_name(lng.second);
	  
	 return (!from_.empty() && !from_name.compare(0,from_.size(),from_)) +
	        (!to_.empty() && !to_name.compare(0,to_.size(),to_));
    }
    std::string from_,to_;
};

int compare_articles(const mt::article& a1,const mt::article& a2)
{
    if (a1.lgk() != a2.lgk())
	return a2.lgk() > a1.lgk();
    else
	return a2.subject() > a1.subject();
}


void show(mt::article_set as)
{
    converter &orig_conv = conv_type::instance(as.langin_);
    converter &tr_conv = conv_type::instance(as.langout_);
    mt::file_map& subj = txtdb_type::instance(mt::datapath+mt::path_separator()+"subjects.txt");
    mt::file_map& spart = txtdb_type::instance(mt::datapath+mt::path_separator()+"speechparts.txt");

    if (as.articles_.empty())
    {
	std::cout<<orig_conv.out(as.untranslated_)<<"  - words not found"<<std::endl;
    }
    else
    {
	std::sort(as.articles_.begin(),as.articles_.end(),compare_articles);
    
	int prev_lgk = -1;
	std::string prev_subject = "x";
	for(size_t i=0;i<as.articles_.size();++i)
	{
	    const mt::article& a = as.articles_[i];
	    if (prev_lgk != a.lgk())
	    {
		std::cout<<std::endl<<orig_conv.out(a.orig())<<","
		<<spart.any_name(mt::to_string<int>(mt::speech_part(a.lgk())))<<std::endl;
		prev_lgk = a.lgk();
		prev_subject = "x";//reset subject
	    }
	    if (prev_subject != a.subject())
	    {
		std::cout<<std::endl<<"\t"<<subj.any_name(a.subject())<<std::endl
		<<"\t\t"<<tr_conv.out(a.translated());
		prev_subject = a.subject();
	    }
	    else
		std::cout<<", "<<tr_conv.out(a.translated());
	}
	std::cout<<std::endl;
    }
}

void do_translate(const std::string& text,mt::lang_code from,mt::lang_code to)
{
    mt::phrase ph;
    mt::fill_phrase(ph,text,from);
    mt::translation tr(ph,from,to);
    std::for_each(tr.asets().begin(), tr.asets().end(), show);
}

/** print program usage information */
void usage(int retcode)
{
	extern const char *__progname;
	std::cout << "Usage: " << __progname << " [-hvft] [--help] [--version] [--from langin] [--to langout] [phrase...]" << std::endl;
	std::cout << "Simple utility to work with Multitran dictionary"<<std::endl;
        std::cout << "This program translates words from one language to another giving results on output."<<std::endl;
	std::cout << "Current supported languages are: 'english' and 'russian'"<<std::endl; 
	std::cout << "\t-h, --help             display help screen" << std::endl;
	std::cout << "\t-v, --version          display version information" << std::endl;
	std::cout << "\t-f, --from             source language name (default english)"<<std::endl;
	std::cout << "\t-t, --to               target language name (default russian)"<<std::endl;
	std::cout << std::endl;
	std::cout << "Report bugs to <inger@altlinux.org>" << std::endl;
	exit(retcode);
}


int main(int argc,char *argv[])
{
	int ret = EXIT_SUCCESS;

	try
	{
		if (1 == argc) usage(EXIT_FAILURE);

		std::string text;
		std::string from_lang,to_lang;

		for(;;)
		{
			static struct option long_options[] =
				{
					{"help", no_argument, 0, 'h'},
					{"version", no_argument, 0, 'v'},
					{"from", no_argument, 0, 'f'},
					{"to", required_argument, 0, 't'},
					{0, 0, 0, 0}
				};
			int c = getopt_long(argc, argv, "hvf:t:", long_options, NULL);
			if ( -1 == c) break;
			switch (c)
			{
			case 'h':
				usage(EXIT_SUCCESS);
			case 'v':
				std::cout << "mtquery version " << VERSION << std::endl;
				std::cout << "Written by Stanislav Ievlev" << std::endl << std::endl;
				std::cout << "Copyright (C) 2004 ALT Linux Team" << std::endl;
				return (EXIT_SUCCESS);
			case 'f':
				from_lang = optarg;
				break;
			case 't':
				to_lang = optarg;
				break;
			default:
				usage(EXIT_FAILURE);
			}
		}
		if (from_lang.empty() && to_lang.empty()) from_lang="english";

		if (optind < argc)
			while (optind < argc) text.append(std::string(" ")+argv[optind++]); //append others to phrase

		mt::linguas avail_langs;

		mt::linguas::iterator lang = std::max_element(avail_langs.begin(),
		                                          avail_langs.end(),
						          compare_names(from_lang,to_lang));
		if (lang == avail_langs.end() ||
		   (!from_lang.empty() && !to_lang.empty() && (compare_names(from_lang,to_lang).distance(*lang)!=2)))
		{
    		    std::cerr<<"illegal language names"<<std::endl;
    		    usage(EXIT_FAILURE);
		}

		//convert to local charset first and do lower
		converter &l2u = conv_type::instance(lang->first);

		do_translate(lower_str(lang->first,l2u.in(text)),
		             lang->first,lang->second);

	}
	catch (std::runtime_error e)
	{
		std::cerr << "fatal:" << e.what() << std::endl;
		ret = EXIT_FAILURE;
	}
	catch (...)
	{
		std::cerr << "unknown exception\n";
		ret = EXIT_FAILURE;
	}
	return ret;
}
